; Cube Image Encoder v1 by Guillermito
; Transforms a 24bits BMP 64x48 into Nokia / Mobiblu image binary format
;
; Imput    : BMP file
; Output   : 384 bytes binary file called bmp_for_cube.bin
;
; Assemble : TASM32 /ml /m3 /z /t cube_image_encoder
; Link     : TLINK32 -Tpe -aa cube_image_encoder,,,import32

callW macro   x
extrn x:PROC
call x
endm

   .486
   .model flat

.data

openfilename_struct:
lStructSize       dd openfilename_struct_size
hwndOwner         dd 0
hInstance         dd 0
lpstrFilter       dd offset filter
lpstrCustomFilter dd 0
nMaxCustFilter    dd 0
nFilterIndex      dd 0
lpstrFile         dd offset namebuffer
nMaxFile          dd 255
lpstrFileTitle    dd 0
nMaxFileTitle     dd 32
lpstrInitialDir   dd 0
lpstrTitle        dd 0
Flags             dd 1000h+4h+200000h
nFileOffset       dw 0
nFileExtension    dw 0
lpstrDefExt       dd 0
lCustData         dd 0
lpfnHook          dd 0
lpTemplateName    dd 0
openfilename_struct_size equ $-offset openfilename_struct

filter            db "Bmp files *.bmp",0,"*.bmp",0,0
namebuffer        db 255 dup(0)
result_title      db "Cube Image Encoder by Guillermito", 0
no_bmp            db "Not a 64x48 pixels 24-bits BMP file",0

file_name          db "bmp_for_cube.bin",0
file_handle2       dd ?
password_or_not    dd ?
file_size          dd ?
file_handle        dd ?
file_mem_buffer    dd ?
file_mem_buffer2   dd ?
file_nb_bytes_read dd ?
results            dd ?
pixels_data_start  dd ?
size_hidden_data   dd ?
y_counter          dd ?
file_size2         dd ?

.code

programme:

;-------- choose a file -------------

push offset openfilename_struct
callW GetOpenFileNameA             ;choose file
test eax, eax
jz end

;-------- open it --------------

push 0
push 80h                          ;FILE_ATTRIBUTE_NORMAL
push 3                            ;OPEN_EXISTING
push 0
push 0
push 80000000h+40000000h          ;GENERIC_READ+GENERIC_WRITE
push dword ptr [lpstrFile]
callW CreateFileA                 ;open file
inc eax
jz end
dec eax
mov file_handle, eax

;---------- get its size ---------

push 0
push dword ptr file_handle
callW GetFileSize                 ;get file size in bytes
inc eax
jz close_file
dec eax
mov file_size, eax

;--------- alloc memory for file --------

mov eax, file_size
shl eax, 1
add eax, 1024
push eax                         ;number of bytes to allocate
push 40h                         ;40h=LMEM_ZEROINIT
callW LocalAlloc
test eax, eax
jz close_file
mov file_mem_buffer, eax
add eax, file_size
add eax, 1024
mov results, eax

;------- read entire file ------------

push 0
push offset file_nb_bytes_read
push file_size
push file_mem_buffer
push file_handle
callW ReadFile
test eax, eax
jz free_memory
mov eax, file_nb_bytes_read
cmp eax, file_size
jnz free_memory

;-------- find BMP signature ---------

mov esi, file_mem_buffer
lodsw
cmp ax, "MB"
jne trouble

;------- find size and number of bits per pixel ------

add esi, 8
lodsd
add eax, file_mem_buffer
mov pixels_data_start, eax
add esi, 4
lodsd
cmp eax, 64d
jne trouble
lodsd
cmp eax, 48d
jne trouble
add esi, 2
lodsw
cmp ax, 24d
je this_looks_like_a_bmp

;-------- not 64x48 or 24-bits BMP -----------

trouble:
push 0
push offset result_title
push offset no_bmp
push 0
callW MessageBoxA
jmp free_memory

;------ Encode BMP into Nokia/Mobiblu format -----------

this_looks_like_a_bmp:

mov eax, file_mem_buffer
add eax, file_size
mov file_mem_buffer2, eax

mov y_counter, 6
mov esi, pixels_data_start
add esi, 3*64*40
mov edi, file_mem_buffer2

parse_image:
mov ecx, 64
push esi

parse_one_block_of_8_lines:
xor eax, eax
cmp byte ptr [esi], 127
jb next1
inc al
next1:
rol al, 1
cmp byte ptr [esi+(64*3)], 127
jb next2
inc al
next2:
rol al, 1
cmp byte ptr [esi+(64*3)*2], 127
jb next3
inc al
next3:
rol al, 1
cmp byte ptr [esi+(64*3)*3], 127
jb next4
inc al
next4:
rol al, 1
cmp byte ptr [esi+(64*3)*4], 127
jb next5
inc al
next5:
rol al, 1
cmp byte ptr [esi+(64*3)*5], 127
jb next6
inc al
next6:
rol al, 1
cmp byte ptr [esi+(64*3)*6], 127
jb next7
inc al
next7:
rol al, 1
cmp byte ptr [esi+(64*3)*7], 127
jb next8
inc al
next8:

stosb
add esi, 3
loop parse_one_block_of_8_lines

next_block:
pop esi
sub esi, 64*3*8 
dec y_counter
jz stop
jmp parse_image

stop:
mov eax, edi
sub eax, file_mem_buffer2
mov file_size2, eax

;------- open a new file on the disk ----------------

push 0                  ;handle of file with attributes to copy (0=Ludwig)
push 80h                ;file attributes (80h=FILE_ATTRIBUTE_NORMAL)
push 2                  ;how to create (2=OPEN_ALWAYS)
push 0                  ;address of security descriptor (0=Ludwig)
push 0                  ;share mode (0=Prevents the file from being shared)
push 40000000h          ;access (read-write) mode (40000000h=GENERIC_WRITE)
push offset file_name   ;address of name of the file
call CreateFileA
mov file_handle2, eax

;------- write buffer in it ----------------

push 0
push offset file_nb_bytes_read          ;address of number of bytes written 
push file_size2                         ;number of bytes to write
push file_mem_buffer2                   ;address of data to write to file
push file_handle2                       ;handle of file to write to
callW WriteFile

;------- close it ----------------

push file_handle2
callW CloseHandle
jmp free_memory

;------- close memory and file ----------------

free_memory:
push file_mem_buffer
callW LocalFree

;----------- exit ----------

close_file:
push dword ptr file_handle
callW CloseHandle

end:
push -1
callW ExitProcess

end programme
